@@ -10,6 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
+from keystoneauth1 import adapter
+import requests
+
 from openstack.baremetal.v1 import _common
 from openstack import exceptions
 from openstack import resource
@@ -30,7 +35,8 @@ class Driver(resource.Resource):
     _query_mapping = resource.QueryParameters(details='detail')
 
     # The BIOS interface fields introduced in 1.40 (Rocky).
-    _max_microversion = '1.40'
+    # The firmware interface fields introduced in 1.86.
+    _max_microversion = '1.86'
 
     #: A list of active hosts that support this driver.
     hosts = resource.Body('hosts', type=list)
@@ -56,6 +62,9 @@ class Driver(resource.Resource):
     #: Default deploy interface implementation.
     #: Introduced in API microversion 1.30.
     default_deploy_interface = resource.Body("default_deploy_interface")
+    #: Default firmware interface implementation.
+    #: Introduced in API microversion 1.86.
+    default_firmware_interface = resource.Body("default_firmware_interface")
     #: Default inspect interface implementation.
     #: Introduced in API microversion 1.30.
     default_inspect_interface = resource.Body("default_inspect_interface")
@@ -95,6 +104,9 @@ class Driver(resource.Resource):
     #: Enabled deploy interface implementations.
     #: Introduced in API microversion 1.30.
     enabled_deploy_interfaces = resource.Body("enabled_deploy_interfaces")
+    #: Enabled firmware interface implementations.
+    #: Introduced in API microversion 1.86.
+    enabled_firmware_interfaces = resource.Body("enabled_firmware_interfaces")
     #: Enabled inspect interface implementations.
     #: Introduced in API microversion 1.30.
     enabled_inspect_interfaces = resource.Body("enabled_inspect_interfaces")
@@ -146,8 +158,12 @@ class Driver(resource.Resource):
         return response.json()
 
     def call_vendor_passthru(
-        self, session, verb: str, method: str, body: dict = None
-    ):
+        self,
+        session: adapter.Adapter,
+        verb: str,
+        method: str,
+        body: ty.Optional[dict] = None,
+    ) -> requests.Response:
         """Call a vendor specific passthru method
 
         Contents of body are params passed to the hardware driver
@@ -164,7 +180,7 @@ class Driver(resource.Resource):
         :returns: response of method call.
         """
         if verb.upper() not in ['GET', 'PUT', 'POST', 'DELETE']:
-            raise ValueError('Invalid verb: {}'.format(verb))
+            raise ValueError(f'Invalid verb: {verb}')
 
         session = self._get_session(session)
         request = self._prepare_request()
@@ -179,8 +195,6 @@ class Driver(resource.Resource):
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
-        msg = "Failed call to method {method} on driver {driver_name}".format(
-            method=method, driver_name=self.name
-        )
+        msg = f"Failed call to method {method} on driver {self.name}"
         exceptions.raise_from_response(response, error_message=msg)
         return response
