@@ -9,6 +9,8 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
@@ -16,6 +18,7 @@ from openstack import utils
 
 class Backup(resource.Resource):
     """Volume Backup"""
+
     resource_key = "backup"
     resources_key = "backups"
     base_path = "/backups"
@@ -24,9 +27,18 @@ class Backup(resource.Resource):
     # search (name~, status~, volume_id~). But this is not documented
     # officially and seem to require microversion be set
     _query_mapping = resource.QueryParameters(
-        'all_tenants', 'limit', 'marker', 'project_id',
-        'name', 'status', 'volume_id',
-        'sort_key', 'sort_dir')
+        "limit",
+        "marker",
+        "offset",
+        "project_id",
+        "name",
+        "status",
+        "volume_id",
+        "sort_key",
+        "sort_dir",
+        "sort",
+        all_projects="all_tenants",
+    )
 
     # capabilities
     allow_fetch = True
@@ -50,6 +62,8 @@ class Backup(resource.Resource):
     data_timestamp = resource.Body('data_timestamp')
     #: backup description
     description = resource.Body("description")
+    #: The UUID of the encryption key. Only included for encrypted volumes.
+    encryption_key_id = resource.Body("encryption_key_id")
     #: Backup fail reason
     fail_reason = resource.Body("fail_reason")
     #: Force backup
@@ -85,6 +99,10 @@ class Backup(resource.Resource):
     user_id = resource.Body('user_id')
     #: The UUID of the volume.
     volume_id = resource.Body("volume_id")
+    #: The name of the volume.
+    volume_name = resource.Body("volume_name")
+
+    _max_microversion = "3.64"
 
     def create(self, session, prepend_key=True, base_path=None, **params):
         """Create a remote resource based on this instance.
@@ -107,35 +125,48 @@ class Backup(resource.Resource):
 
         session = self._get_session(session)
         microversion = self._get_microversion(session, action='create')
-        requires_id = (self.create_requires_id
-                       if self.create_requires_id is not None
-                       else self.create_method == 'PUT')
+        requires_id = (
+            self.create_requires_id
+            if self.create_requires_id is not None
+            else self.create_method == 'PUT'
+        )
 
         if self.create_exclude_id_from_body:
             self._body._dirty.discard("id")
 
         if self.create_method == 'POST':
-            request = self._prepare_request(requires_id=requires_id,
-                                            prepend_key=prepend_key,
-                                            base_path=base_path)
+            request = self._prepare_request(
+                requires_id=requires_id,
+                prepend_key=prepend_key,
+                base_path=base_path,
+            )
             # NOTE(gtema) this is a funny example of when attribute
             # is called "incremental" on create, "is_incremental" on get
             # and use of "alias" or "aka" is not working for such conflict,
             # since our preferred attr name is exactly "is_incremental"
             body = request.body
             if 'is_incremental' in body['backup']:
-                body['backup']['incremental'] = \
-                    body['backup'].pop('is_incremental')
-            response = session.post(request.url,
-                                    json=request.body, headers=request.headers,
-                                    microversion=microversion, params=params)
+                body['backup']['incremental'] = body['backup'].pop(
+                    'is_incremental'
+                )
+            response = session.post(
+                request.url,
+                json=request.body,
+                headers=request.headers,
+                microversion=microversion,
+                params=params,
+            )
         else:
             # Just for safety of the implementation (since PUT removed)
             raise exceptions.ResourceFailure(
-                "Invalid create method: %s" % self.create_method)
+                f"Invalid create method: {self.create_method}"
+            )
 
-        has_body = (self.has_body if self.create_returns_body is None
-                    else self.create_returns_body)
+        has_body = (
+            self.has_body
+            if self.create_returns_body is None
+            else self.create_returns_body
+        )
         self.microversion = microversion
         self._translate_response(response, has_body=has_body)
         # direct comparision to False since we need to rule out None
@@ -147,8 +178,9 @@ class Backup(resource.Resource):
     def _action(self, session, body, microversion=None):
         """Preform backup actions given the message body."""
         url = utils.urljoin(self.base_path, self.id, 'action')
-        resp = session.post(url, json=body,
-                            microversion=self._max_microversion)
+        resp = session.post(
+            url, json=body, microversion=self._max_microversion
+        )
         exceptions.raise_from_response(resp)
         return resp
 
@@ -161,27 +193,26 @@ class Backup(resource.Resource):
         :return: Updated backup instance
         """
         url = utils.urljoin(self.base_path, self.id, "restore")
-        body = {'restore': {}}
+        body: dict[str, dict] = {'restore': {}}
         if volume_id:
             body['restore']['volume_id'] = volume_id
         if name:
             body['restore']['name'] = name
         if not (volume_id or name):
-            raise exceptions.SDKException('Either of `name` or `volume_id`'
-                                          ' must be specified.')
+            raise exceptions.SDKException(
+                'Either of `name` or `volume_id` must be specified.'
+            )
         response = session.post(url, json=body)
-        self._translate_response(response, has_body=False)
+        self._translate_response(response, resource_response_key='restore')
         return self
 
     def force_delete(self, session):
-        """Force backup deletion
-        """
-        body = {'os-force_delete': {}}
+        """Force backup deletion"""
+        body = {'os-force_delete': None}
         self._action(session, body)
 
     def reset(self, session, status):
-        """Reset the status of the backup
-        """
+        """Reset the status of the backup"""
         body = {'os-reset_status': {'status': status}}
         self._action(session, body)
 
