@@ -14,40 +14,18 @@
 
 import contextlib
 import fnmatch
-import functools
 import inspect
+import ipaddress
 import re
-import time
+import socket
 import uuid
 
 from decorator import decorator
 import jmespath
-import netifaces
-import sre_constants
+import psutil
 
 from openstack import _log
-from openstack.cloud import exc
-
-
-_decorated_methods = []
-
-
-def _make_unicode(input):
-    """Turn an input into unicode unconditionally
-
-    :param input: A unicode, string or other object
-    """
-    try:
-        if isinstance(input, unicode):
-            return input
-        if isinstance(input, str):
-            return input.decode('utf-8')
-        else:
-            # int, for example
-            return unicode(input)
-    except NameError:
-        # python3!
-        return str(input)
+from openstack import exceptions
 
 
 def _dictify_resource(resource):
@@ -71,16 +49,12 @@ def _filter_list(data, name_or_id, filters):
     :param filters: A dictionary of meta data to use for further filtering.
         Elements of this dictionary may, themselves, be dictionaries. Example::
 
-            {
-                'last_name': 'Smith',
-                'other': {
-                    'gender': 'Female'
-                }
-            }
+            {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
         OR
 
         A string containing a jmespath expression for further filtering.
+        Invalid filters will be ignored.
     """
     # The logger is openstack.cloud.fmmatch to allow a user/operator to
     # configure logging not to communicate about fnmatch misses
@@ -88,23 +62,24 @@ def _filter_list(data, name_or_id, filters):
     log = _log.setup_logging('openstack.fnmatch')
     if name_or_id:
         # name_or_id might already be unicode
-        name_or_id = _make_unicode(name_or_id)
+        name_or_id = str(name_or_id)
         identifier_matches = []
         bad_pattern = False
         try:
             fn_reg = re.compile(fnmatch.translate(name_or_id))
-        except sre_constants.error:
+        except re.error:
             # If the fnmatch re doesn't compile, then we don't care,
             # but log it in case the user DID pass a pattern but did
             # it poorly and wants to know what went wrong with their
             # search
             fn_reg = None
         for e in data:
-            e_id = _make_unicode(e.get('id', None))
-            e_name = _make_unicode(e.get('name', None))
+            e_id = str(e.get('id', None))
+            e_name = str(e.get('name', None))
 
-            if ((e_id and e_id == name_or_id)
-                    or (e_name and e_name == name_or_id)):
+            if (e_id and e_id == name_or_id) or (
+                e_name and e_name == name_or_id
+            ):
                 identifier_matches.append(e)
             else:
                 # Only try fnmatch if we don't match exactly
@@ -113,8 +88,9 @@ def _filter_list(data, name_or_id, filters):
                     # so that we log the bad pattern
                     bad_pattern = True
                     continue
-                if ((e_id and fn_reg.match(e_id))
-                        or (e_name and fn_reg.match(e_name))):
+                if (e_id and fn_reg.match(e_id)) or (
+                    e_name and fn_reg.match(e_name)
+                ):
                     identifier_matches.append(e)
         if not identifier_matches and bad_pattern:
             log.debug("Bad pattern passed to fnmatch", exc_info=True)
@@ -130,6 +106,17 @@ def _filter_list(data, name_or_id, filters):
         if not d:
             return False
         for key in f.keys():
+            if key not in d:
+                log.warning(
+                    "Invalid filter: %s is not an attribute of %s.%s",
+                    key,
+                    e.__class__.__module__,
+                    e.__class__.__qualname__,
+                )
+                # we intentionally skip this since the user was trying to
+                # filter on _something_, but we don't know what that
+                # _something_ was
+                raise AttributeError(key)
             if isinstance(f[key], dict):
                 if not _dict_filter(f[key], d.get(key, None)):
                     return False
@@ -139,15 +126,8 @@ def _filter_list(data, name_or_id, filters):
 
     filtered = []
     for e in data:
-        filtered.append(e)
-        for key in filters.keys():
-            if isinstance(filters[key], dict):
-                if not _dict_filter(filters[key], e.get(key, None)):
-                    filtered.pop()
-                    break
-            elif e.get(key, None) != filters[key]:
-                filtered.pop()
-                break
+        if _dict_filter(filters, e):
+            filtered.append(e)
     return filtered
 
 
@@ -167,31 +147,36 @@ def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
         Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
     """
 
-    # Sometimes in the control flow of shade, we already have an object
+    # Sometimes in the control flow of openstacksdk, we already have an object
     # fetched. Rather than then needing to pull the name or id out of that
     # object, pass it in here and rely on caching to prevent us from making
     # an additional call, it's simple enough to test to see if we got an
     # object and just short-circuit return it.
 
-    if (hasattr(name_or_id, 'id')
-            or (isinstance(name_or_id, dict) and 'id' in name_or_id)):
+    if hasattr(name_or_id, 'id') or (
+        isinstance(name_or_id, dict) and 'id' in name_or_id
+    ):
         return name_or_id
 
     # If a uuid is passed short-circuit it calling the
     # get_<resource_name>_by_id method
     if getattr(cloud, 'use_direct_get', False) and _is_uuid_like(name_or_id):
-        get_resource = getattr(cloud, 'get_%s_by_id' % resource, None)
+        get_resource = getattr(cloud, f'get_{resource}_by_id', None)
         if get_resource:
             return get_resource(name_or_id)
 
-    search = resource if callable(resource) else getattr(
-        cloud, 'search_%ss' % resource, None)
+    search = (
+        resource
+        if callable(resource)
+        else getattr(cloud, f'search_{resource}s', None)
+    )
     if search:
         entities = search(name_or_id, filters, **kwargs)
         if entities:
             if len(entities) > 1:
-                raise exc.OpenStackCloudException(
-                    "Multiple matches found for %s" % name_or_id)
+                raise exceptions.SDKException(
+                    f"Multiple matches found for {name_or_id}"
+                )
             return entities[0]
     return None
 
@@ -199,15 +184,19 @@ def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
 def localhost_supports_ipv6():
     """Determine whether the local host supports IPv6
 
-    We look for a default route that supports the IPv6 address family,
-    and assume that if it is present, this host has globally routable
-    IPv6 connectivity.
+    We look for the all ip addresses configured to this node, and assume that
+    if any of these is IPv6 address (but not loopback or link local), this host
+    has IPv6 connectivity.
     """
 
-    try:
-        return netifaces.AF_INET6 in netifaces.gateways()['default']
-    except AttributeError:
-        return False
+    for ifname, if_addrs in psutil.net_if_addrs().items():
+        for if_addr in if_addrs:
+            if if_addr.family != socket.AF_INET6:
+                continue
+            addr = ipaddress.ip_address(if_addr.address)
+            if not addr.is_link_local and not addr.is_loopback:
+                return True
+    return False
 
 
 def valid_kwargs(*valid_args):
@@ -230,70 +219,37 @@ def valid_kwargs(*valid_args):
         for k in kwargs:
             if k not in argspec.args[1:] and k not in valid_args:
                 raise TypeError(
-                    "{f}() got an unexpected keyword argument "
-                    "'{arg}'".format(f=inspect.stack()[1][3], arg=k))
+                    f"{inspect.stack()[1][3]}() got an unexpected keyword argument "
+                    f"'{k}'"
+                )
         return func(*args, **kwargs)
+
     return func_wrapper
 
 
-def _func_wrap(f):
-    # NOTE(morgan): This extra wrapper is intended to eliminate ever
-    # passing a bound method to dogpile.cache's cache_on_arguments. In
-    # 0.7.0 and later it is impossible to pass bound methods to the
-    # decorator. This was introduced when utilizing the decorate module in
-    # lieu of a direct wrap implementation.
-    @functools.wraps(f)
-    def inner(*args, **kwargs):
-        return f(*args, **kwargs)
-    return inner
-
-
-def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
-    _cache_name = cache_on_kwargs.pop('resource', None)
-
-    def _inner_cache_on_arguments(func):
-        def _cache_decorator(obj, *args, **kwargs):
-            the_method = obj._get_cache(_cache_name).cache_on_arguments(
-                *cache_on_args, **cache_on_kwargs)(
-                    _func_wrap(func.__get__(obj, type(obj))))
-            return the_method(*args, **kwargs)
-
-        def invalidate(obj, *args, **kwargs):
-            return obj._get_cache(
-                _cache_name).cache_on_arguments()(func).invalidate(
-                    *args, **kwargs)
-
-        _cache_decorator.invalidate = invalidate
-        _cache_decorator.func = func
-        _decorated_methods.append(func.__name__)
-
-        return _cache_decorator
-    return _inner_cache_on_arguments
-
-
 @contextlib.contextmanager
-def shade_exceptions(error_message=None):
-    """Context manager for dealing with shade exceptions.
+def openstacksdk_exceptions(error_message=None):
+    """Context manager for dealing with openstack exceptions.
 
     :param string error_message: String to use for the exception message
-        content on non-OpenStackCloudExceptions.
+        content on non-SDKException exception.
 
-        Useful for avoiding wrapping shade OpenStackCloudException exceptions
+        Useful for avoiding wrapping SDKException exceptions
         within themselves. Code called from within the context may throw such
         exceptions without having to catch and reraise them.
 
-        Non-OpenStackCloudException exceptions thrown within the context will
+        Non-SDKException exceptions thrown within the context will
         be wrapped and the exception message will be appended to the given
         error message.
     """
     try:
         yield
-    except exc.OpenStackCloudException:
+    except exceptions.SDKException:
         raise
     except Exception as e:
         if error_message is None:
             error_message = str(e)
-        raise exc.OpenStackCloudException(error_message)
+        raise exceptions.SDKException(error_message)
 
 
 def safe_dict_min(key, data):
@@ -318,10 +274,9 @@ def safe_dict_min(key, data):
             try:
                 val = int(d[key])
             except ValueError:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Search for minimum value failed. "
-                    "Value for {key} is not an integer: {value}".format(
-                        key=key, value=d[key])
+                    f"Value for {key} is not an integer: {d[key]}"
                 )
             if (min_value is None) or (val < min_value):
                 min_value = val
@@ -350,69 +305,15 @@ def safe_dict_max(key, data):
             try:
                 val = int(d[key])
             except ValueError:
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Search for maximum value failed. "
-                    "Value for {key} is not an integer: {value}".format(
-                        key=key, value=d[key])
+                    f"Value for {key} is not an integer: {d[key]}"
                 )
             if (max_value is None) or (val > max_value):
                 max_value = val
     return max_value
 
 
-def _call_client_and_retry(client, url, retry_on=None,
-                           call_retries=3, retry_wait=2,
-                           **kwargs):
-    """Method to provide retry operations.
-
-    Some APIs utilize HTTP errors on certain operations to indicate that
-    the resource is presently locked, and as such this mechanism provides
-    the ability to retry upon known error codes.
-
-    :param object client: The client method, such as:
-        ``self.baremetal_client.post``
-    :param string url: The URL to perform the operation upon.
-    :param integer retry_on: A list of error codes that can be retried on.
-        The method also supports a single integer to be
-        defined.
-    :param integer call_retries: The number of times to retry the call upon
-        the error code defined by the 'retry_on' parameter. Default: 3
-    :param integer retry_wait: The time in seconds to wait between retry
-        attempts. Default: 2
-
-    :returns: The object returned by the client call.
-    """
-
-    # NOTE(TheJulia): This method, as of this note, does not have direct
-    # unit tests, although is fairly well tested by the tests checking
-    # retry logic in test_baremetal_node.py.
-    log = _log.setup_logging('shade.http')
-
-    if isinstance(retry_on, int):
-        retry_on = [retry_on]
-
-    count = 0
-    while (count < call_retries):
-        count += 1
-        try:
-            ret_val = client(url, **kwargs)
-        except exc.OpenStackCloudHTTPError as e:
-            if (retry_on is not None
-                    and e.response.status_code in retry_on):
-                log.debug('Received retryable error %(err)s, waiting '
-                          '%(wait)s seconds to retry', {
-                              'err': e.response.status_code,
-                              'wait': retry_wait
-                          })
-                time.sleep(retry_wait)
-                continue
-            else:
-                raise
-        # Break out of the loop, since the loop should only continue
-        # when we encounter a known connection error.
-        return ret_val
-
-
 def parse_range(value):
     """Parse a numerical range string.
 
@@ -456,7 +357,8 @@ def range_filter(data, key, range_exp):
     :param string range_exp: The expression describing the range of values.
 
     :returns: A list subset of the original data set.
-    :raises: OpenStackCloudException on invalid range expressions.
+    :raises: :class:`~openstack.exceptions.SDKException` on invalid range
+        expressions.
     """
     filtered = []
     range_exp = str(range_exp).upper()
@@ -484,8 +386,7 @@ def range_filter(data, key, range_exp):
 
     # If parsing the range fails, it must be a bad value.
     if val_range is None:
-        raise exc.OpenStackCloudException(
-            "Invalid range value: {value}".format(value=range_exp))
+        raise exceptions.SDKException(f"Invalid range value: {range_exp}")
 
     op = val_range[0]
     if op:
@@ -524,9 +425,7 @@ def generate_patches_from_kwargs(operation, **kwargs):
     """
     patches = []
     for k, v in kwargs.items():
-        patch = {'op': operation,
-                 'value': v,
-                 'path': '/%s' % k}
+        patch = {'op': operation, 'value': v, 'path': f'/{k}'}
         patches.append(patch)
     return sorted(patches)
 
@@ -569,11 +468,13 @@ class FileSegment:
 
 
 def _format_uuid_string(string):
-    return (string.replace('urn:', '')
-                  .replace('uuid:', '')
-                  .strip('{}')
-                  .replace('-', '')
-                  .lower())
+    return (
+        string.replace('urn:', '')
+        .replace('uuid:', '')
+        .strip('{}')
+        .replace('-', '')
+        .lower()
+    )
 
 
 def _is_uuid_like(val):
