@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.clustering.v1 import action as _action
 from openstack.clustering.v1 import build_info
 from openstack.clustering.v1 import cluster as _cluster
@@ -28,6 +30,21 @@ from openstack import resource
 
 
 class Proxy(proxy.Proxy):
+    _resource_registry = {
+        "action": _action.Action,
+        "build_info": build_info.BuildInfo,
+        "cluster": _cluster.Cluster,
+        "cluster_attr": _cluster_attr.ClusterAttr,
+        "cluster_policy": _cluster_policy.ClusterPolicy,
+        "event": _event.Event,
+        "node": _node.Node,
+        "policy": _policy.Policy,
+        "policy_type": _policy_type.PolicyType,
+        "profile": _profile.Profile,
+        "profile_type": _profile_type.ProfileType,
+        "receiver": _receiver.Receiver,
+        "service": _service.Service,
+    }
 
     def get_build_info(self):
         """Get build info for service engine and API
@@ -53,7 +70,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A :class:`~openstack.clustering.v1.profile_type.ProfileType`
             object.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             profile_type matching the name could be found.
         """
         return self._get(_profile_type.ProfileType, profile_type)
@@ -74,7 +91,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A :class:`~openstack.clustering.v1.policy_type.PolicyType`
             object.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             policy_type matching the name could be found.
         """
         return self._get(_policy_type.PolicyType, policy_type)
@@ -97,7 +114,7 @@ class Proxy(proxy.Proxy):
         :param profile: The value can be either the name or ID of a profile or
             a :class:`~openstack.clustering.v1.profile.Profile` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the profile could not be found. When set to ``True``, no exception
             will be raised when attempting to delete a non-existent profile.
 
@@ -110,15 +127,16 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The name or ID of a profile.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
         :returns: One :class:`~openstack.clustering.v1.profile.Profile` object
             or None
         """
-        return self._find(_profile.Profile, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _profile.Profile, name_or_id, ignore_missing=ignore_missing
+        )
 
     def get_profile(self, profile):
         """Get a single profile.
@@ -127,7 +145,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.clustering.v1.profile.Profile` instance.
 
         :returns: One :class:`~openstack.clustering.v1.profile.Profile`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             profile matching the criteria could be found.
         """
         return self._get(_profile.Profile, profile)
@@ -203,7 +221,7 @@ class Proxy(proxy.Proxy):
         :param cluster: The value can be either the name or ID of a cluster or
             a :class:`~openstack.cluster.v1.cluster.Cluster` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the cluster could not be found. When set to ``True``, no exception
             will be raised when attempting to delete a non-existent cluster.
         :param bool force_delete: When set to ``True``, the cluster deletion
@@ -216,23 +234,25 @@ class Proxy(proxy.Proxy):
             server = self._get_resource(_cluster.Cluster, cluster)
             return server.force_delete(self)
         else:
-            return self._delete(_cluster.Cluster, cluster,
-                                ignore_missing=ignore_missing)
+            return self._delete(
+                _cluster.Cluster, cluster, ignore_missing=ignore_missing
+            )
 
     def find_cluster(self, name_or_id, ignore_missing=True):
         """Find a single cluster.
 
         :param str name_or_id: The name or ID of a cluster.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
         :returns: One :class:`~openstack.clustering.v1.cluster.Cluster` object
             or None
         """
-        return self._find(_cluster.Cluster, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _cluster.Cluster, name_or_id, ignore_missing=ignore_missing
+        )
 
     def get_cluster(self, cluster):
         """Get a single cluster.
@@ -241,7 +261,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.clustering.v1.cluster.Cluster` instance.
 
         :returns: One :class:`~openstack.clustering.v1.cluster.Cluster`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             cluster matching the criteria could be found.
         """
         return self._get(_cluster.Cluster, cluster)
@@ -480,8 +500,9 @@ class Proxy(proxy.Proxy):
 
         :returns: A dictionary containing the list of attribute values.
         """
-        return self._list(_cluster_attr.ClusterAttr, cluster_id=cluster,
-                          path=path)
+        return self._list(
+            _cluster_attr.ClusterAttr, cluster_id=cluster, path=path
+        )
 
     def check_cluster(self, cluster, **params):
         """Check a cluster.
@@ -541,7 +562,7 @@ class Proxy(proxy.Proxy):
         :param node: The value can be either the name or ID of a node or a
             :class:`~openstack.cluster.v1.node.Node` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the node could not be found. When set to ``True``, no exception
             will be raised when attempting to delete a non-existent node.
         :param bool force_delete: When set to ``True``, the node deletion
@@ -554,23 +575,25 @@ class Proxy(proxy.Proxy):
             server = self._get_resource(_node.Node, node)
             return server.force_delete(self)
         else:
-            return self._delete(_node.Node, node,
-                                ignore_missing=ignore_missing)
+            return self._delete(
+                _node.Node, node, ignore_missing=ignore_missing
+            )
 
     def find_node(self, name_or_id, ignore_missing=True):
         """Find a single node.
 
         :param str name_or_id: The name or ID of a node.
         :param bool ignore_missing: When set to "False"
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the specified node does not exist.
             when set to "True", None will be returned when
             attempting to find a nonexistent policy
         :returns: One :class:`~openstack.clustering.v1.node.Node` object
             or None.
         """
-        return self._find(_node.Node, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _node.Node, name_or_id, ignore_missing=ignore_missing
+        )
 
     def get_node(self, node, details=False):
         """Get a single node.
@@ -581,7 +604,7 @@ class Proxy(proxy.Proxy):
             server should return more details when retrieving the node data.
 
         :returns: One :class:`~openstack.clustering.v1.node.Node`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             node matching the name or ID could be found.
         """
         # NOTE: When retrieving node with details (using NodeDetail resource),
@@ -720,7 +743,7 @@ class Proxy(proxy.Proxy):
         :param policy: The value can be either the name or ID of a policy or a
             :class:`~openstack.clustering.v1.policy.Policy` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the policy could not be found. When set to ``True``, no exception
             will be raised when attempting to delete a non-existent policy.
 
@@ -733,15 +756,16 @@ class Proxy(proxy.Proxy):
 
         :param str name_or_id: The name or ID of a policy.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the specified policy does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent policy.
         :returns: A policy object or None.
         :rtype: :class:`~openstack.clustering.v1.policy.Policy`
         """
-        return self._find(_policy.Policy, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _policy.Policy, name_or_id, ignore_missing=ignore_missing
+        )
 
     def get_policy(self, policy):
         """Get a single policy.
@@ -751,7 +775,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A policy object.
         :rtype: :class:`~openstack.clustering.v1.policy.Policy`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             policy matching the criteria could be found.
         """
         return self._get(_policy.Policy, policy)
@@ -819,8 +843,9 @@ class Proxy(proxy.Proxy):
         :returns: A generator of cluster-policy binding instances.
         """
         cluster_id = resource.Resource._get_id(cluster)
-        return self._list(_cluster_policy.ClusterPolicy, cluster_id=cluster_id,
-                          **query)
+        return self._list(
+            _cluster_policy.ClusterPolicy, cluster_id=cluster_id, **query
+        )
 
     def get_cluster_policy(self, cluster_policy, cluster):
         """Get a cluster-policy binding.
@@ -833,11 +858,12 @@ class Proxy(proxy.Proxy):
 
         :returns: a cluster-policy binding object.
         :rtype: :class:`~openstack.clustering.v1.cluster_policy.CLusterPolicy`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             cluster-policy binding matching the criteria could be found.
         """
-        return self._get(_cluster_policy.ClusterPolicy, cluster_policy,
-                         cluster_id=cluster)
+        return self._get(
+            _cluster_policy.ClusterPolicy, cluster_policy, cluster_id=cluster
+        )
 
     def create_receiver(self, **attrs):
         """Create a new receiver from attributes.
@@ -869,29 +895,31 @@ class Proxy(proxy.Proxy):
         :param receiver: The value can be either the name or ID of a receiver
             or a :class:`~openstack.clustering.v1.receiver.Receiver` instance.
         :param bool ignore_missing: When set to ``False``, an exception
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the receiver could not be found. When set to ``True``, no exception
             will be raised when attempting to delete a non-existent receiver.
 
         :returns: ``None``
         """
-        self._delete(_receiver.Receiver, receiver,
-                     ignore_missing=ignore_missing)
+        self._delete(
+            _receiver.Receiver, receiver, ignore_missing=ignore_missing
+        )
 
     def find_receiver(self, name_or_id, ignore_missing=True):
         """Find a single receiver.
 
         :param str name_or_id: The name or ID of a receiver.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the specified receiver does not exist. When
             set to ``True``, None will be returned when attempting to
             find a nonexistent receiver.
         :returns: A receiver object or None.
         :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`
         """
-        return self._find(_receiver.Receiver, name_or_id,
-                          ignore_missing=ignore_missing)
+        return self._find(
+            _receiver.Receiver, name_or_id, ignore_missing=ignore_missing
+        )
 
     def get_receiver(self, receiver):
         """Get a single receiver.
@@ -901,7 +929,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A receiver object.
         :rtype: :class:`~openstack.clustering.v1.receiver.Receiver`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             receiver matching the criteria could be found.
         """
         return self._get(_receiver.Receiver, receiver)
@@ -934,7 +962,7 @@ class Proxy(proxy.Proxy):
 
         :returns: an action object.
         :rtype: :class:`~openstack.clustering.v1.action.Action`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             action matching the criteria could be found.
         """
         return self._get(_action.Action, action)
@@ -985,7 +1013,7 @@ class Proxy(proxy.Proxy):
 
         :returns: an event object.
         :rtype: :class:`~openstack.clustering.v1.event.Event`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             event matching the criteria could be found.
         """
         return self._get(_event.Event, event)
@@ -1020,47 +1048,6 @@ class Proxy(proxy.Proxy):
         """
         return self._list(_event.Event, **query)
 
-    def wait_for_status(self, res, status, failures=None, interval=2,
-                        wait=120):
-        """Wait for a resource to be in a particular status.
-
-        :param res: The resource to wait on to reach the specified status.
-            The resource must have a ``status`` attribute.
-        :type resource: A :class:`~openstack.resource.Resource` object.
-        :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
-        :type failures: :py:class:`list`
-        :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to the desired status failed to occur in specified seconds.
-        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
-            has transited to one of the failure statuses.
-        :raises: :class:`~AttributeError` if the resource does not have a
-            ``status`` attribute.
-        """
-        failures = [] if failures is None else failures
-        return resource.wait_for_status(
-            self, res, status, failures, interval, wait)
-
-    def wait_for_delete(self, res, interval=2, wait=120):
-        """Wait for a resource to be deleted.
-
-        :param res: The resource to wait on to be deleted.
-        :type resource: A :class:`~openstack.resource.Resource` object.
-        :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to delete failed to occur in the specified seconds.
-        """
-        return resource.wait_for_delete(self, res, interval, wait)
-
     def services(self, **query):
         """Get a generator of services.
 
@@ -1078,8 +1065,71 @@ class Proxy(proxy.Proxy):
 
         :returns: A :class:`~openstack.clustering.v1.profile_type.ProfileType`
             object.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             profile_type matching the name could be found.
         """
         obj = self._get_resource(_profile_type.ProfileType, profile_type)
         return obj.type_ops(self)
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
