@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
 from openstack.common import metadata
 from openstack.common import tag
 from openstack.compute.v2 import flavor
@@ -19,6 +22,15 @@ from openstack import resource
 from openstack import utils
 
 
+# Workaround Python's lack of an undefined sentinel
+# https://python-patterns.guide/python/sentinel-object/
+class Unset:
+    def __bool__(self) -> ty.Literal[False]:
+        return False
+
+
+UNSET: Unset = Unset()
+
 CONSOLE_TYPE_ACTION_MAPPING = {
     'novnc': 'os-getVNCConsole',
     'xvpvnc': 'os-getVNCConsole',
@@ -74,6 +86,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         "vm_state",
         "sort_key",
         "sort_dir",
+        "pinned_availability_zone",
         access_ipv4="access_ip_v4",
         access_ipv6="access_ip_v6",
         has_config_drive="config_drive",
@@ -89,7 +102,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         **tag.TagMixin._tag_query_parameters,
     )
 
-    _max_microversion = '2.91'
+    _max_microversion = '2.96'
 
     #: A list of dictionaries holding links relevant to this server.
     links = resource.Body('links')
@@ -144,6 +157,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: A fault object. Only available when the server status
     #: is ERROR or DELETED and a fault occurred.
     fault = resource.Body('fault')
+    #: The host to boot the server on.
+    host = resource.Body('host')
     #: The host status.
     host_status = resource.Body('host_status')
     #: The hostname set on the instance when it is booted.
@@ -166,7 +181,6 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: routable ipv6 addresses, and to private_ipv4 if the Connection was
     #: created with private=True. Otherwise it will be set to public_ipv4.
     interface_ip = resource.Computed('interface_ip', default='')
-
     # The locked status of the server
     is_locked = resource.Body('locked', type=bool)
     #: The UUID of the kernel image when using an AMI. Will be null if not.
@@ -180,6 +194,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     launch_index = resource.Body('OS-EXT-SRV-ATTR:launch_index', type=int)
     #: The timestamp when the server was launched.
     launched_at = resource.Body('OS-SRV-USG:launched_at')
+    #: The reason the server was locked, if any.
+    locked_reason = resource.Body('locked_reason')
     #: The maximum number of servers to create.
     max_count = resource.Body('max_count')
     #: The minimum number of servers to create.
@@ -189,6 +205,14 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: networks parameter, the server attaches to the only network
     #: created for the current tenant.
     networks = resource.Body('networks')
+    #: Personality files. This should be a list of dicts with each dict
+    #: containing a file name ('name') and a base64-encoded file contents
+    #: ('contents')
+    personality = resource.Body('personality', type=list)
+    #: The availability zone requested during server creation OR pinned
+    #: availability zone, which is configured using default_schedule_zone
+    #: config option.
+    pinned_availability_zone = resource.Body('pinned_availability_zone')
     #: The power state of this server.
     power_state = resource.Body('OS-EXT-STS:power_state')
     #: While the server is building, this value represents the percentage
@@ -257,13 +281,18 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         requires_id=True,
         prepend_key=True,
+        patch=False,
         base_path=None,
+        params=None,
         **kwargs,
     ):
-        request = super(Server, self)._prepare_request(
+        request = super()._prepare_request(
             requires_id=requires_id,
             prepend_key=prepend_key,
+            patch=patch,
             base_path=base_path,
+            params=params,
+            **kwargs,
         )
 
         server_body = request.body[self.resource_key]
@@ -315,7 +344,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         exceptions.raise_from_response(response)
         return response
 
-    def change_password(self, session, password):
+    def change_password(self, session, password, *, microversion=None):
         """Change the administrator password to the given password.
 
         :param session: The session to use for making this request.
@@ -323,22 +352,40 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :returns: None
         """
         body = {'changePassword': {'adminPass': password}}
-        self._action(session, body)
+        self._action(session, body, microversion=microversion)
 
-    def get_password(self, session):
+    def get_password(self, session, *, microversion=None):
         """Get the encrypted administrator password.
 
         :param session: The session to use for making this request.
         :returns: The encrypted administrator password.
         """
         url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
 
-        response = session.get(url)
+        response = session.get(url, microversion=microversion)
         exceptions.raise_from_response(response)
 
         data = response.json()
         return data.get('password')
 
+    def clear_password(self, session, *, microversion=None):
+        """Clear the administrator password.
+
+        This removes the password from the database. It does not actually
+        change the server password.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
+        url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session, action='commit')
+
+        response = session.delete(url, microversion=microversion)
+        exceptions.raise_from_response(response)
+
     def reboot(self, session, reboot_type):
         """Reboot server where reboot_type might be 'SOFT' or 'HARD'.
 
@@ -362,14 +409,17 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         session,
         image,
-        name=None,
-        admin_password=None,
-        preserve_ephemeral=None,
-        access_ipv4=None,
-        access_ipv6=None,
-        metadata=None,
-        user_data=None,
-        key_name=None,
+        name=UNSET,
+        admin_password=UNSET,
+        preserve_ephemeral=UNSET,
+        access_ipv4=UNSET,
+        access_ipv6=UNSET,
+        metadata=UNSET,
+        user_data=UNSET,
+        key_name=UNSET,
+        description=UNSET,
+        trusted_image_certificates=UNSET,
+        hostname=UNSET,
     ):
         """Rebuild the server with the given arguments.
 
@@ -388,25 +438,38 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param metadata: Metadata to set on the updated server. (Optional)
         :param user_data: User data to set on the updated server. (Optional)
         :param key_name: A key name to set on the updated server. (Optional)
+        :param description: The description to set on the updated server.
+            (Optional) (Requires API microversion 2.19)
+        :param trusted_image_certificates: The trusted image certificates to
+            set on the updated server. (Optional) (Requires API microversion
+            2.78)
+        :param hostname: The hostname to set on the updated server. (Optional)
+            (Requires API microversion 2.90)
         :returns: The updated server.
         """
         action = {'imageRef': resource.Resource._get_id(image)}
-        if preserve_ephemeral is not None:
+        if preserve_ephemeral is not UNSET:
             action['preserve_ephemeral'] = preserve_ephemeral
-        if name is not None:
+        if name is not UNSET:
             action['name'] = name
-        if admin_password is not None:
+        if admin_password is not UNSET:
             action['adminPass'] = admin_password
-        if access_ipv4 is not None:
+        if access_ipv4 is not UNSET:
             action['accessIPv4'] = access_ipv4
-        if access_ipv6 is not None:
+        if access_ipv6 is not UNSET:
             action['accessIPv6'] = access_ipv6
-        if metadata is not None:
+        if metadata is not UNSET:
             action['metadata'] = metadata
-        if user_data is not None:
+        if user_data is not UNSET:
             action['user_data'] = user_data
-        if key_name is not None:
+        if key_name is not UNSET:
             action['key_name'] = key_name
+        if description is not UNSET:
+            action['description'] = description
+        if trusted_image_certificates is not UNSET:
+            action['trusted_image_certificates'] = trusted_image_certificates
+        if hostname is not UNSET:
+            action['hostname'] = hostname
 
         body = {'rebuild': action}
         response = self._action(session, body)
@@ -474,14 +537,13 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             microversion = '2.45'
         response = self._action(session, body, microversion)
 
-        body = None
         try:
-            # There might be body, might be not
-            body = response.json()
+            # There might be a body, there might not be
+            response_body = response.json()
         except Exception:
-            pass
-        if body and 'image_id' in body:
-            image_id = body['image_id']
+            response_body = None
+        if response_body and 'image_id' in response_body:
+            image_id = response_body['image_id']
         else:
             image_id = response.headers['Location'].rsplit('/', 1)[1]
 
@@ -634,7 +696,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param locked_reason: The reason for locking the server.
         :returns: None
         """
-        body = {"lock": None}
+        body: dict[str, ty.Any] = {"lock": None}
         if locked_reason is not None:
             body["lock"] = {
                 "locked_reason": locked_reason,
@@ -662,7 +724,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             provided, the server will use the existing image. (Optional)
         :returns: None
         """
-        body = {"rescue": {}}
+        body: dict[str, ty.Any] = {"rescue": {}}
         if admin_pass is not None:
             body["rescue"]["adminPass"] = admin_pass
         if image_ref is not None:
@@ -680,7 +742,14 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         body = {"unrescue": None}
         self._action(session, body)
 
-    def evacuate(self, session, host=None, admin_pass=None, force=None):
+    def evacuate(
+        self,
+        session,
+        host=None,
+        admin_pass=None,
+        force=None,
+        on_shared_storage=None,
+    ):
         """Evacuate the server.
 
         :param session: The session to use for making this request.
@@ -688,15 +757,19 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param admin_pass: The admin password to set on the evacuated instance.
             (Optional)
         :param force: Whether to force evacuation.
+        :param on_shared_storage: Whether the host is using shared storage.
+            (Optional) (Only supported before microversion 2.14)
         :returns: None
         """
-        body = {"evacuate": {}}
+        body: dict[str, ty.Any] = {"evacuate": {}}
         if host is not None:
             body["evacuate"]["host"] = host
         if admin_pass is not None:
             body["evacuate"]["adminPass"] = admin_pass
         if force is not None:
             body["evacuate"]["force"] = force
+        if on_shared_storage is not None:
+            body["evacuate"]["onSharedStorage"] = on_shared_storage
         self._action(session, body)
 
     def start(self, session):
@@ -769,13 +842,22 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         body = {'unshelve': data or None}
         self._action(session, body)
 
-    def migrate(self, session):
+    def migrate(self, session, *, host=None):
         """Migrate the server.
 
         :param session: The session to use for making this request.
+        :param host: The host to migrate the server to. (Optional)
         :returns: None
         """
-        body = {"migrate": None}
+        if host and not utils.supports_microversion(session, '2.56'):
+            raise ValueError(
+                "The 'host' option is only supported on microversion 2.56 or "
+                "greater."
+            )
+
+        body: dict[str, ty.Any] = {"migrate": None}
+        if host:
+            body["migrate"] = {"host": host}
         self._action(session, body)
 
     def trigger_crash_dump(self, session):
@@ -795,7 +877,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             (Optional)
         :returns: None
         """
-        body = {"os-getConsoleOutput": {}}
+        body: dict[str, ty.Any] = {"os-getConsoleOutput": {}}
         if length is not None:
             body["os-getConsoleOutput"]["length"] = length
         resp = self._action(session, body)
@@ -812,7 +894,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         """
         action = CONSOLE_TYPE_ACTION_MAPPING.get(console_type)
         if not action:
-            raise ValueError("Unsupported console type %s" % console_type)
+            raise ValueError(f"Unsupported console type {console_type}")
         body = {action: {'type': console_type}}
         resp = self._action(session, body)
         return resp.json().get('console')
@@ -885,12 +967,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
@@ -907,13 +989,13 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         disk_over_commit,
     ):
         microversion = None
-        body = {
+        body: dict[str, ty.Any] = {
             'host': None,
         }
         if block_migration == 'auto':
             raise ValueError(
-                "Live migration on this cloud does not support 'auto' as"
-                " a parameter to block_migration, but only True and False."
+                "Live migration on this cloud does not support 'auto' as "
+                "a parameter to block_migration, but only True and False."
             )
         body['block_migration'] = block_migration or False
         body['disk_over_commit'] = disk_over_commit or False
@@ -921,12 +1003,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
@@ -940,7 +1022,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         :param session: The session to use for making this request.
         :returns: None
         """
-        utils.require_microversion(session, 2.78)
+        utils.require_microversion(session, '2.78')
 
         url = utils.urljoin(Server.base_path, self.id, 'topology')
 
