@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.identity import identity_service
 from openstack import resource
 
 
@@ -18,18 +17,23 @@ class Policy(resource.Resource):
     resource_key = 'policy'
     resources_key = 'policies'
     base_path = '/policies'
-    service = identity_service.IdentityService()
 
     # capabilities
     allow_create = True
-    allow_retrieve = True
-    allow_update = True
+    allow_fetch = True
+    allow_commit = True
     allow_delete = True
     allow_list = True
-    patch_update = True
+    commit_method = 'PATCH'
 
     # Properties
     #: The policy rule set itself, as a serialized blob. *Type: string*
-    blob = resource.prop('blob')
+    blob = resource.Body('blob')
+    #: The links for the policy resource.
+    links = resource.Body('links')
+    #: The ID for the project.
+    project_id = resource.Body('project_id')
     #: The MIME Media Type of the serialized policy blob. *Type: string*
-    type = resource.prop('type')
+    type = resource.Body('type')
+    #: The ID of the user who owns the policy
+    user_id = resource.Body('user_id')
