@@ -9,7 +9,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+from openstack.common import tag
 from openstack import exceptions
 from openstack import resource
 from openstack import utils
@@ -29,7 +29,14 @@ class Stack(resource.Resource):
     allow_delete = True
 
     _query_mapping = resource.QueryParameters(
-        'resolve_outputs'
+        'action',
+        'name',
+        'status',
+        'project_id',
+        'owner_id',
+        'username',
+        project_id='tenant_id',
+        **tag.TagMixin._tag_query_parameters,
     )
 
     # Properties
@@ -84,7 +91,7 @@ class Stack(resource.Resource):
     #: A text explaining how the stack transits to its current status.
     status_reason = resource.Body('stack_status_reason')
     #: A list of strings used as tags on the stack
-    tags = resource.Body('tags')
+    tags = resource.Body('tags', type=list, default=[])
     #: A dict containing the template use for stack creation.
     template = resource.Body('template', type=dict)
     #: Stack template description text. Currently contains the same text
@@ -105,45 +112,48 @@ class Stack(resource.Resource):
     #: The ID of the user project created for this stack.
     user_project_id = resource.Body('stack_user_project_id')
 
-    def create(self, session, base_path=None):
+    def create(self, session, prepend_key=False, *args, **kwargs):
         # This overrides the default behavior of resource creation because
         # heat doesn't accept resource_key in its request.
-        return super(Stack, self).create(session, prepend_key=False,
-                                         base_path=base_path)
+        return super().create(session, prepend_key, *args, **kwargs)
 
-    def commit(self, session, base_path=None):
-        # This overrides the default behavior of resource creation because
-        # heat doesn't accept resource_key in its request.
-        return super(Stack, self).commit(session, prepend_key=False,
-                                         has_body=False, base_path=None)
-
-    def update(self, session, preview=False):
+    def commit(
+        self,
+        session,
+        prepend_key=True,
+        has_body=True,
+        retry_on_conflict=None,
+        base_path=None,
+        *,
+        microversion=None,
+        preview=False,
+        **kwargs,
+    ):
         # This overrides the default behavior of resource update because
         # we need to use other endpoint for update preview.
         base_path = None
         if self.name and self.id:
-            base_path = '/stacks/%(stack_name)s/%(stack_id)s' % {
-                'stack_name': self.name,
-                'stack_id': self.id}
+            base_path = f'/stacks/{self.name}/{self.id}'
         elif self.name or self.id:
             # We have only one of name/id. Do not try to build a stacks/NAME/ID
             # path
-            base_path = '/stacks/%(stack_identity)s' % {
-                'stack_identity': self.name or self.id}
+            base_path = f'/stacks/{self.name or self.id}'
         request = self._prepare_request(
-            prepend_key=False,
-            requires_id=False,
-            base_path=base_path)
+            prepend_key=False, requires_id=False, base_path=base_path
+        )
 
-        microversion = self._get_microversion_for(session, 'commit')
+        microversion = self._get_microversion(session, action='commit')
 
         request_url = request.url
         if preview:
             request_url = utils.urljoin(request_url, 'preview')
 
         response = session.put(
-            request_url, json=request.body, headers=request.headers,
-            microversion=microversion)
+            request_url,
+            json=request.body,
+            headers=request.headers,
+            microversion=microversion,
+        )
 
         self.microversion = microversion
         self._translate_response(response, has_body=True)
@@ -152,35 +162,79 @@ class Stack(resource.Resource):
     def _action(self, session, body):
         """Perform stack actions"""
         url = utils.urljoin(self.base_path, self._get_id(self), 'actions')
-        resp = session.post(url, json=body)
-        return resp.json()
+        resp = session.post(url, json=body, microversion=self.microversion)
+        exceptions.raise_from_response(resp)
+        return resp
 
     def check(self, session):
         return self._action(session, {'check': ''})
 
     def abandon(self, session):
-        url = utils.urljoin(self.base_path, self.name,
-                            self._get_id(self), 'abandon')
+        url = utils.urljoin(
+            self.base_path, self.name, self._get_id(self), 'abandon'
+        )
         resp = session.delete(url)
         return resp.json()
 
-    def fetch(self, session, requires_id=True,
-              base_path=None, error_message=None, resolve_outputs=True):
+    def export(self, session):
+        """Export a stack data
 
+        :param session: The session to use for making this request.
+        :return: A dictionary containing the stack data.
+        """
+        url = utils.urljoin(
+            self.base_path, self.name, self._get_id(self), 'export'
+        )
+        resp = session.get(url)
+        exceptions.raise_from_response(resp)
+        return resp.json()
+
+    def suspend(self, session):
+        """Suspend a stack
+
+        :param session: The session to use for making this request
+        :returns: None
+        """
+        body = {"suspend": None}
+        self._action(session, body)
+
+    def resume(self, session):
+        """Resume a stack
+
+        :param session: The session to use for making this request
+        :returns: None
+        """
+        body = {"resume": None}
+        self._action(session, body)
+
+    def fetch(
+        self,
+        session,
+        requires_id=True,
+        base_path=None,
+        error_message=None,
+        skip_cache=False,
+        *,
+        resolve_outputs=True,
+        **params,
+    ):
         if not self.allow_fetch:
             raise exceptions.MethodNotSupported(self, "fetch")
 
-        request = self._prepare_request(requires_id=requires_id,
-                                        base_path=base_path)
+        request = self._prepare_request(
+            requires_id=requires_id, base_path=base_path
+        )
         # session = self._get_session(session)
-        microversion = self._get_microversion_for(session, 'fetch')
+        microversion = self._get_microversion(session, action='fetch')
 
         # NOTE(gtema): would be nice to simply use QueryParameters, however
         # Heat return 302 with parameters being set into URL and requests
         # apply parameters again, what results in them being set doubled
         if not resolve_outputs:
             request.url = request.url + '?resolve_outputs=False'
-        response = session.get(request.url, microversion=microversion)
+        response = session.get(
+            request.url, microversion=microversion, skip_cache=skip_cache
+        )
         kwargs = {}
         if error_message:
             kwargs['error_message'] = error_message
@@ -189,8 +243,7 @@ class Stack(resource.Resource):
         self._translate_response(response, **kwargs)
 
         if self and self.status in ['DELETE_COMPLETE', 'ADOPT_COMPLETE']:
-            raise exceptions.ResourceNotFound(
-                "No stack found for %s" % self.id)
+            raise exceptions.NotFoundException(f"No stack found for {self.id}")
         return self
 
     @classmethod
@@ -202,7 +255,7 @@ class Stack(resource.Resource):
         :param name_or_id: This resource's identifier, if needed by
                            the request. The default is ``None``.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    :class:`~openstack.exceptions.NotFoundException` will be
                     raised when the resource does not exist.
                     When set to ``True``, None will be returned when
                     attempting to find a nonexistent resource.
@@ -215,16 +268,15 @@ class Stack(resource.Resource):
                  or None if nothing matches.
         :raises: :class:`openstack.exceptions.DuplicateResource` if more
                  than one resource is found for this request.
-        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
+        :raises: :class:`openstack.exceptions.NotFoundException` if nothing
                  is found and ignore_missing is ``False``.
         """
         session = cls._get_session(session)
         # Try to short-circuit by looking directly for a matching ID.
         try:
             match = cls.existing(
-                id=name_or_id,
-                connection=session._get_connection(),
-                **params)
+                id=name_or_id, connection=session._get_connection(), **params
+            )
             return match.fetch(session, **params)
         except exceptions.NotFoundException:
             pass
@@ -234,8 +286,9 @@ class Stack(resource.Resource):
 
         if ignore_missing:
             return None
-        raise exceptions.ResourceNotFound(
-            "No %s found for %s" % (cls.__name__, name_or_id))
+        raise exceptions.NotFoundException(
+            f"No {cls.__name__} found for {name_or_id}"
+        )
 
 
 StackPreview = Stack
