@@ -14,6 +14,7 @@ import functools
 import typing as ty
 import urllib
 from urllib.parse import urlparse
+import warnings
 
 try:
     import simplejson
@@ -28,9 +29,11 @@ from keystoneauth1 import adapter
 from openstack import _log
 from openstack import exceptions
 from openstack import resource
+from openstack import utils
+from openstack import warnings as os_warnings
 
-
-ResourceType = ty.TypeVar('ResourceType', bound=resource.Resource)
+if ty.TYPE_CHECKING:
+    from openstack import connection
 
 
 # The _check_resource decorator is used on Proxy methods to ensure that
@@ -49,13 +52,12 @@ def _check_resource(strict=False):
                 and actual is not None
                 and not isinstance(actual, resource.Resource)
             ):
-                raise ValueError("A %s must be passed" % expected.__name__)
+                raise ValueError(f"A {expected.__name__} must be passed")
             elif isinstance(actual, resource.Resource) and not isinstance(
                 actual, expected
             ):
                 raise ValueError(
-                    "Expected %s but received %s"
-                    % (expected.__name__, actual.__class__.__name__)
+                    f"Expected {expected.__name__} but received {actual.__class__.__name__}"
                 )
 
             return method(self, expected, actual, *args, **kwargs)
@@ -74,19 +76,21 @@ def normalize_metric_name(name):
 class Proxy(adapter.Adapter):
     """Represents a service."""
 
-    retriable_status_codes = None
+    retriable_status_codes: ty.Optional[list[int]] = None
     """HTTP status codes that should be retried by default.
 
     The number of retries is defined by the configuration in parameters called
     ``<service-type>_status_code_retries``.
     """
 
-    _resource_registry: ty.Dict[str, ty.Type[resource.Resource]] = {}
+    _resource_registry: dict[str, type[resource.Resource]] = {}
     """Registry of the supported resourses.
 
     Dictionary of resource names (key) types (value).
     """
 
+    _connection: 'connection.Connection'
+
     def __init__(
         self,
         session,
@@ -104,7 +108,9 @@ class Proxy(adapter.Adapter):
         kwargs.setdefault(
             'retriable_status_codes', self.retriable_status_codes
         )
-        super().__init__(session=session, *args, **kwargs)
+        # TODO(stephenfin): Resolve this by copying the signature of
+        # adapter.Adapter.__init__
+        super().__init__(session=session, *args, **kwargs)  # type: ignore
         self._statsd_client = statsd_client
         self._statsd_prefix = statsd_prefix
         self._prometheus_counter = prometheus_counter
@@ -112,7 +118,7 @@ class Proxy(adapter.Adapter):
         self._influxdb_client = influxdb_client
         self._influxdb_config = influxdb_config
         if self.service_type:
-            log_name = 'openstack.{0}'.format(self.service_type)
+            log_name = f'openstack.{self.service_type}'
         else:
             log_name = 'openstack'
         self.log = _log.setup_logging(log_name)
@@ -224,6 +230,12 @@ class Proxy(adapter.Adapter):
           /servers/{id}/os-security-groups -> ['server', 'os-security-groups']
           /v2.0/networks.json -> ['networks']
         """
+        if service_type is not None:
+            warnings.warn(
+                "The 'service_type' parameter is unnecesary and will be "
+                "removed in a future release.",
+                os_warnings.RemovedInSDK60Warning,
+            )
 
         url_path = urllib.parse.urlparse(url).path.strip()
         # Remove / from the beginning to keep the list indexes of interesting
@@ -332,14 +344,14 @@ class Proxy(adapter.Adapter):
             with self._statsd_client.pipeline() as pipe:
                 if response is not None:
                     duration = int(response.elapsed.total_seconds() * 1000)
-                    metric_name = '%s.%s' % (key, str(response.status_code))
+                    metric_name = f'{key}.{str(response.status_code)}'
                     pipe.timing(metric_name, duration)
                     pipe.incr(metric_name)
                     if duration > 1000:
-                        pipe.incr('%s.over_1000' % key)
+                        pipe.incr(f'{key}.over_1000')
                 elif exc is not None:
-                    pipe.incr('%s.failed' % key)
-                pipe.incr('%s.attempted' % key)
+                    pipe.incr(f'{key}.failed')
+                pipe.incr(f'{key}.attempted')
         except Exception:
             # We do not want errors in metric reporting ever break client
             self.log.exception("Exception reporting metrics")
@@ -352,8 +364,8 @@ class Proxy(adapter.Adapter):
         if response is not None and not method:
             method = response.request.method
         parsed_url = urlparse(url)
-        endpoint = "{}://{}{}".format(
-            parsed_url.scheme, parsed_url.netloc, parsed_url.path
+        endpoint = (
+            f"{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}"
         )
         if response is not None:
             labels = dict(
@@ -395,7 +407,7 @@ class Proxy(adapter.Adapter):
             tags['status_code'] = str(response.status_code)
             # Note(gtema): emit also status_code as a value (counter)
             fields[str(response.status_code)] = 1
-            fields['%s.%s' % (method, response.status_code)] = 1
+            fields[f'{method}.{response.status_code}'] = 1
             # Note(gtema): status_code field itself is also very helpful on the
             # graphs to show what was the code, instead of counting its
             # occurences
@@ -410,7 +422,7 @@ class Proxy(adapter.Adapter):
             else 'openstack_api'
         )
         # Note(gtema) append service name into the measurement name
-        measurement = '%s.%s' % (measurement, self.service_type)
+        measurement = f'{measurement}.{self.service_type}'
         data = [dict(measurement=measurement, tags=tags, fields=fields)]
         try:
             self._influxdb_client.write_points(data)
@@ -429,8 +441,11 @@ class Proxy(adapter.Adapter):
         )
 
     def _get_resource(
-        self, resource_type: ty.Type[ResourceType], value, **attrs
-    ) -> ResourceType:
+        self,
+        resource_type: type[resource.ResourceT],
+        value: ty.Union[None, str, resource.ResourceT, utils.Munch],
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
@@ -476,18 +491,47 @@ class Proxy(adapter.Adapter):
             value = resource.Resource._get_id(parent)
         return value
 
+    @ty.overload
     def _find(
         self,
-        resource_type: ty.Type[ResourceType],
-        name_or_id,
-        ignore_missing=True,
-        **attrs,
-    ) -> ty.Optional[ResourceType]:
+        resource_type: type[resource.ResourceT],
+        name_or_id: str,
+        ignore_missing: ty.Literal[True] = True,
+        **attrs: ty.Any,
+    ) -> ty.Optional[resource.ResourceT]: ...
+
+    @ty.overload
+    def _find(
+        self,
+        resource_type: type[resource.ResourceT],
+        name_or_id: str,
+        ignore_missing: ty.Literal[False],
+        **attrs: ty.Any,
+    ) -> resource.ResourceT: ...
+
+    # excuse the duplication here: it's mypy's fault
+    # https://github.com/python/mypy/issues/14764
+    @ty.overload
+    def _find(
+        self,
+        resource_type: type[resource.ResourceT],
+        name_or_id: str,
+        ignore_missing: bool,
+        **attrs: ty.Any,
+    ) -> ty.Optional[resource.ResourceT]: ...
+
+    def _find(
+        self,
+        resource_type: type[resource.ResourceT],
+        name_or_id: str,
+        ignore_missing: bool = True,
+        **attrs: ty.Any,
+    ) -> ty.Optional[resource.ResourceT]:
         """Find a resource
 
         :param name_or_id: The name or ID of a resource to find.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -504,11 +548,11 @@ class Proxy(adapter.Adapter):
     @_check_resource(strict=False)
     def _delete(
         self,
-        resource_type: ty.Type[ResourceType],
-        value,
-        ignore_missing=True,
-        **attrs,
-    ) -> ty.Optional[ResourceType]:
+        resource_type: type[resource.ResourceT],
+        value: ty.Union[str, resource.ResourceT],
+        ignore_missing: bool = True,
+        **attrs: ty.Any,
+    ) -> ty.Optional[resource.ResourceT]:
         """Delete a resource
 
         :param resource_type: The type of resource to delete. This should
@@ -518,7 +562,7 @@ class Proxy(adapter.Adapter):
             resource or a :class:`~openstack.resource.Resource`
             subclass.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent resource.
@@ -528,7 +572,7 @@ class Proxy(adapter.Adapter):
         :raises: ``ValueError`` if ``value`` is a
             :class:`~openstack.resource.Resource` that doesn't match
             the ``resource_type``.
-            :class:`~openstack.exceptions.ResourceNotFound` when
+            :class:`~openstack.exceptions.NotFoundException` when
             ignore_missing if ``False`` and a nonexistent resource
             is attempted to be deleted.
         """
@@ -536,7 +580,7 @@ class Proxy(adapter.Adapter):
 
         try:
             rv = res.delete(self)
-        except exceptions.ResourceNotFound:
+        except exceptions.NotFoundException:
             if ignore_missing:
                 return None
             raise
@@ -546,11 +590,11 @@ class Proxy(adapter.Adapter):
     @_check_resource(strict=False)
     def _update(
         self,
-        resource_type: ty.Type[ResourceType],
-        value,
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: ty.Union[str, resource.ResourceT],
+        base_path: ty.Optional[str] = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Update a resource
 
         :param resource_type: The type of resource to update.
@@ -576,10 +620,10 @@ class Proxy(adapter.Adapter):
 
     def _create(
         self,
-        resource_type: ty.Type[ResourceType],
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        base_path: ty.Optional[str] = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -599,16 +643,23 @@ class Proxy(adapter.Adapter):
         :returns: The result of the ``create``
         :rtype: :class:`~openstack.resource.Resource`
         """
+        # Check for attributes whose names conflict with the parameters
+        # specified in the method.
+        conflicting_attrs = attrs.get('__conflicting_attrs', {})
+        if conflicting_attrs:
+            for k, v in conflicting_attrs.items():
+                attrs[k] = v
+            attrs.pop('__conflicting_attrs')
         conn = self._get_connection()
         res = resource_type.new(connection=conn, **attrs)
         return res.create(self, base_path=base_path)
 
     def _bulk_create(
         self,
-        resource_type: ty.Type[ResourceType],
-        data,
-        base_path=None,
-    ) -> ty.Generator[ResourceType, None, None]:
+        resource_type: type[resource.ResourceT],
+        data: list[dict[str, ty.Any]],
+        base_path: ty.Optional[str] = None,
+    ) -> ty.Generator[resource.ResourceT, None, None]:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -631,13 +682,13 @@ class Proxy(adapter.Adapter):
     @_check_resource(strict=False)
     def _get(
         self,
-        resource_type: ty.Type[ResourceType],
-        value=None,
-        requires_id=True,
-        base_path=None,
-        skip_cache=False,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: ty.Union[str, resource.ResourceT, None] = None,
+        requires_id: bool = True,
+        base_path: ty.Optional[str] = None,
+        skip_cache: bool = False,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Fetch a resource
 
         :param resource_type: The type of resource to get.
@@ -667,19 +718,17 @@ class Proxy(adapter.Adapter):
             requires_id=requires_id,
             base_path=base_path,
             skip_cache=skip_cache,
-            error_message="No {resource_type} found for {value}".format(
-                resource_type=resource_type.__name__, value=value
-            ),
+            error_message=f"No {resource_type.__name__} found for {value}",
         )
 
     def _list(
         self,
-        resource_type: ty.Type[ResourceType],
-        paginated=True,
-        base_path=None,
-        jmespath_filters=None,
-        **attrs,
-    ) -> ty.Generator[ResourceType, None, None]:
+        resource_type: type[resource.ResourceT],
+        paginated: bool = True,
+        base_path: ty.Optional[str] = None,
+        jmespath_filters: ty.Optional[str] = None,
+        **attrs: ty.Any,
+    ) -> ty.Generator[resource.ResourceT, None, None]:
         """List a resource
 
         :param resource_type: The type of resource to list. This should
@@ -705,6 +754,14 @@ class Proxy(adapter.Adapter):
             :class:`~openstack.resource.Resource` that doesn't match
             the ``resource_type``.
         """
+        # Check for attributes whose names conflict with the parameters
+        # specified in the method.
+        conflicting_attrs = attrs.get('__conflicting_attrs', {})
+        if conflicting_attrs:
+            for k, v in conflicting_attrs.items():
+                attrs[k] = v
+            attrs.pop('__conflicting_attrs')
+
         data = resource_type.list(
             self, paginated=paginated, base_path=base_path, **attrs
         )
@@ -716,11 +773,11 @@ class Proxy(adapter.Adapter):
 
     def _head(
         self,
-        resource_type: ty.Type[ResourceType],
-        value=None,
-        base_path=None,
-        **attrs,
-    ) -> ResourceType:
+        resource_type: type[resource.ResourceT],
+        value: ty.Union[str, resource.ResourceT, None] = None,
+        base_path: ty.Optional[str] = None,
+        **attrs: ty.Any,
+    ) -> resource.ResourceT:
         """Retrieve a resource's header
 
         :param resource_type: The type of resource to retrieve.
@@ -821,8 +878,8 @@ class Proxy(adapter.Adapter):
                         # There are filters set, but we can't get required
                         # attribute, so skip the resource
                         self.log.debug(
-                            'Requested cleanup attribute %s is not '
-                            'available on the resource' % k
+                            f'Requested cleanup attribute {k} is not '
+                            'available on the resource'
                         )
                         part_cond.append(False)
                 except Exception:
@@ -836,11 +893,15 @@ class Proxy(adapter.Adapter):
         if resource is None or skip_resources is None:
             return False
 
+        if self.service_type is None:
+            # to keep mypy happy - this should never happen
+            return False
+
         resource_name = f"{self.service_type.replace('-', '_')}.{resource}"
 
         if resource_name in skip_resources:
             self.log.debug(
-                f"Skipping resource {resource_name} " "in project cleanup"
+                f"Skipping resource {resource_name} in project cleanup"
             )
             return True
 
