@@ -12,6 +12,8 @@
 
 import operator
 import os
+import time
+import uuid
 
 from keystoneauth1 import discover
 
@@ -39,28 +41,48 @@ def _disable_keep_alive(conn):
 
 
 class BaseFunctionalTest(base.TestCase):
+    user_cloud: connection.Connection
+    user_cloud_alt: connection.Connection
+    operator_cloud: connection.Connection
 
     _wait_for_timeout_key = ''
 
     def setUp(self):
-        super(BaseFunctionalTest, self).setUp()
+        super().setUp()
         self.conn = connection.Connection(config=TEST_CLOUD_REGION)
         _disable_keep_alive(self.conn)
 
         self._demo_name = os.environ.get('OPENSTACKSDK_DEMO_CLOUD', 'devstack')
+        if not self._demo_name:
+            raise self.failureException(
+                "OPENSTACKSDK_OPERATOR_CLOUD must be set to a non-empty value"
+            )
+
         self._demo_name_alt = os.environ.get(
-            'OPENSTACKSDK_DEMO_CLOUD_ALT', 'devstack-alt',
+            'OPENSTACKSDK_DEMO_CLOUD_ALT',
+            'devstack-alt',
         )
+        if not self._demo_name_alt:
+            raise self.failureException(
+                "OPENSTACKSDK_OPERATOR_CLOUD must be set to a non-empty value"
+            )
+
         self._op_name = os.environ.get(
-            'OPENSTACKSDK_OPERATOR_CLOUD', 'devstack-admin',
+            'OPENSTACKSDK_OPERATOR_CLOUD',
+            'devstack-admin',
         )
+        if not self._op_name:
+            raise self.failureException(
+                "OPENSTACKSDK_OPERATOR_CLOUD must be set to a non-empty value"
+            )
 
         self.config = openstack.config.OpenStackConfig()
         self._set_user_cloud()
         self._set_operator_cloud()
 
-        self.identity_version = \
-            self.operator_cloud.config.get_api_version('identity')
+        self.identity_version = self.user_cloud.config.get_api_version(
+            'identity'
+        )
 
         self.flavor = self._pick_flavor()
         self.image = self._pick_image()
@@ -68,18 +90,20 @@ class BaseFunctionalTest(base.TestCase):
         # Defines default timeout for wait_for methods used
         # in the functional tests
         self._wait_for_timeout = int(
-            os.getenv(self._wait_for_timeout_key, os.getenv(
-                'OPENSTACKSDK_FUNC_TEST_TIMEOUT', 300)))
+            os.getenv(
+                self._wait_for_timeout_key,
+                os.getenv('OPENSTACKSDK_FUNC_TEST_TIMEOUT', 300),
+            )
+        )
 
     def _set_user_cloud(self, **kwargs):
         user_config = self.config.get_one(cloud=self._demo_name, **kwargs)
         self.user_cloud = connection.Connection(config=user_config)
         _disable_keep_alive(self.user_cloud)
 
-        # This cloud is used by the project_cleanup test, so you can't rely on
-        # it
         user_config_alt = self.config.get_one(
-            cloud=self._demo_name_alt, **kwargs)
+            cloud=self._demo_name_alt, **kwargs
+        )
         self.user_cloud_alt = connection.Connection(config=user_config_alt)
         _disable_keep_alive(self.user_cloud_alt)
 
@@ -98,7 +122,7 @@ class BaseFunctionalTest(base.TestCase):
             return None
 
         flavors = self.user_cloud.list_flavors(get_extra=False)
-        self.add_info_on_exception('flavors', flavors)
+        # self.add_info_on_exception('flavors', flavors)
 
         flavor_name = os.environ.get('OPENSTACKSDK_FLAVOR')
 
@@ -111,7 +135,8 @@ class BaseFunctionalTest(base.TestCase):
                     return flavor
 
             raise self.failureException(
-                "Cloud does not have flavor '%s'", flavor_name,
+                "Cloud does not have flavor '%s'",
+                flavor_name,
             )
 
         # Enable running functional tests against RAX, which requires
@@ -138,7 +163,7 @@ class BaseFunctionalTest(base.TestCase):
             return None
 
         images = self.user_cloud.list_images()
-        self.add_info_on_exception('images', images)
+        # self.add_info_on_exception('images', images)
 
         image_name = os.environ.get('OPENSTACKSDK_IMAGE')
 
@@ -151,7 +176,8 @@ class BaseFunctionalTest(base.TestCase):
                     return image
 
             raise self.failureException(
-                "Cloud does not have image '%s'", image_name,
+                "Cloud does not have image '%s'",
+                image_name,
             )
 
         for image in images:
@@ -178,6 +204,7 @@ class BaseFunctionalTest(base.TestCase):
         def cleanup():
             result = func(*args, **kwargs)
             self.assertIsNone(result)
+
         self.addCleanup(cleanup)
 
     def require_service(self, service_type, min_microversion=None, **kwargs):
@@ -193,14 +220,14 @@ class BaseFunctionalTest(base.TestCase):
         :returns: True if the service exists, otherwise False.
         """
         if not self.conn.has_service(service_type):
-            self.skipTest('Service {service_type} not found in cloud'.format(
-                service_type=service_type))
+            self.skipTest(f'Service {service_type} not found in cloud')
 
         if not min_microversion:
             return
 
         data = self.conn.session.get_endpoint_data(
-            service_type=service_type, **kwargs)
+            service_type=service_type, **kwargs
+        )
 
         if not (
             data.min_microversion
@@ -216,11 +243,58 @@ class BaseFunctionalTest(base.TestCase):
                 f'{min_microversion}'
             )
 
+    def getUniqueString(self, prefix=None):
+        """Generate unique resource name"""
+        # Globally unique names can only rely on some form of uuid
+        # unix_t is also used to easier determine orphans when running real
+        # functional tests on a real cloud
+        return (
+            prefix if prefix else ''
+        ) + f"{int(time.time())}-{uuid.uuid4().hex}"
+
+    def create_temporary_project(self):
+        """Create a new temporary project.
+
+        This is useful for tests that modify things like quotas, which would
+        cause issues for other tests.
+        """
+        project_name = self.getUniqueString('project-')
+        project = self.operator_cloud.get_project(project_name)
+        if not project:
+            params = {
+                'name': project_name,
+                'description': f'Temporary project created for {self.id()}',
+                # assume identity API v3 for now
+                'domain_id': self.operator_cloud.get_domain('default')['id'],
+            }
+            project = self.operator_cloud.create_project(**params)
+
+        # Grant the current user access to the project
+        user_id = self.operator_cloud.current_user_id
+        role_assignment = self.operator_cloud.list_role_assignments(
+            {'user': user_id, 'project': project['id']}
+        )
+        if not role_assignment:
+            self.operator_cloud.grant_role(
+                'member', user=user_id, project=project['id'], wait=True
+            )
+
+        self.addCleanup(self._delete_temporary_project, project)
+
+        return project
+
+    def _delete_temporary_project(self, project):
+        self.operator_cloud.revoke_role(
+            'member',
+            user=self.operator_cloud.current_user_id,
+            project=project.id,
+        )
+        self.operator_cloud.delete_project(project.id)
+
 
 class KeystoneBaseFunctionalTest(BaseFunctionalTest):
-
     def setUp(self):
-        super(KeystoneBaseFunctionalTest, self).setUp()
+        super().setUp()
 
         use_keystone_v2 = os.environ.get('OPENSTACKSDK_USE_KEYSTONE_V2', False)
         if use_keystone_v2:
