@@ -11,27 +11,32 @@
 # under the License.
 
 from unittest import mock
+import warnings
 
 from openstack.block_storage.v3 import _proxy
 from openstack.block_storage.v3 import backup
 from openstack.block_storage.v3 import capabilities
 from openstack.block_storage.v3 import extension
 from openstack.block_storage.v3 import group
+from openstack.block_storage.v3 import group_snapshot
 from openstack.block_storage.v3 import group_type
-from openstack.block_storage.v3 import limits
+from openstack.block_storage.v3 import quota_class_set
 from openstack.block_storage.v3 import quota_set
 from openstack.block_storage.v3 import resource_filter
+from openstack.block_storage.v3 import service
 from openstack.block_storage.v3 import snapshot
 from openstack.block_storage.v3 import stats
 from openstack.block_storage.v3 import type
 from openstack.block_storage.v3 import volume
-from openstack import resource
+from openstack.identity.v3 import project
+from openstack import proxy as proxy_base
 from openstack.tests.unit import test_proxy_base
+from openstack import warnings as os_warnings
 
 
 class TestVolumeProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestVolumeProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -40,7 +45,15 @@ class TestVolume(TestVolumeProxy):
         self.verify_get(self.proxy.get_volume, volume.Volume)
 
     def test_volume_find(self):
-        self.verify_find(self.proxy.find_volume, volume.Volume)
+        self.verify_find(
+            self.proxy.find_volume,
+            volume.Volume,
+            method_kwargs={'all_projects': True},
+            expected_kwargs={
+                "list_base_path": "/volumes/detail",
+                "all_projects": True,
+            },
+        )
 
     def test_volumes_detailed(self):
         self.verify_list(
@@ -113,7 +126,16 @@ class TestVolume(TestVolumeProxy):
         self.verify_wait_for_status(
             self.proxy.wait_for_status,
             method_args=[value],
-            expected_args=[self.proxy, value, 'available', ['error'], 2, 120],
+            expected_args=[
+                self.proxy,
+                value,
+                'available',
+                ['error'],
+                2,
+                None,
+                'status',
+                None,
+            ],
         )
 
 
@@ -124,11 +146,13 @@ class TestPools(TestVolumeProxy):
 
 class TestLimit(TestVolumeProxy):
     def test_limits_get(self):
-        self.verify_get(
+        self._verify(
+            'openstack.resource.Resource.fetch',
             self.proxy.get_limits,
-            limits.Limit,
             method_args=[],
-            expected_kwargs={'requires_id': False},
+            method_kwargs={'project': 'foo'},
+            expected_args=[self.proxy],
+            expected_kwargs={'requires_id': False, 'project_id': 'foo'},
         )
 
 
@@ -149,7 +173,11 @@ class TestGroup(TestVolumeProxy):
         self.verify_get(self.proxy.get_group, group.Group)
 
     def test_group_find(self):
-        self.verify_find(self.proxy.find_group, group.Group)
+        self.verify_find(
+            self.proxy.find_group,
+            group.Group,
+            expected_kwargs={'list_base_path': '/groups/detail'},
+        )
 
     def test_groups(self):
         self.verify_list(self.proxy.groups, group.Group)
@@ -181,6 +209,59 @@ class TestGroup(TestVolumeProxy):
         self._verify(self.proxy.reset_group_state, group.Group)
 
 
+class TestGroupSnapshot(TestVolumeProxy):
+    def test_group_snapshot_get(self):
+        self.verify_get(
+            self.proxy.get_group_snapshot, group_snapshot.GroupSnapshot
+        )
+
+    def test_group_snapshot_find(self):
+        self.verify_find(
+            self.proxy.find_group_snapshot,
+            group_snapshot.GroupSnapshot,
+            expected_kwargs={
+                'list_base_path': '/group_snapshots/detail',
+            },
+        )
+
+    def test_group_snapshots(self):
+        self.verify_list(
+            self.proxy.group_snapshots,
+            group_snapshot.GroupSnapshot,
+            expected_kwargs={},
+        )
+
+    def test_group_snapshots__detailed(self):
+        self.verify_list(
+            self.proxy.group_snapshots,
+            group_snapshot.GroupSnapshot,
+            method_kwargs={'details': True, 'query': 1},
+            expected_kwargs={
+                'query': 1,
+                'base_path': '/group_snapshots/detail',
+            },
+        )
+
+    def test_group_snapshot_create(self):
+        self.verify_create(
+            self.proxy.create_group_snapshot, group_snapshot.GroupSnapshot
+        )
+
+    def test_group_snapshot_delete(self):
+        self.verify_delete(
+            self.proxy.delete_group_snapshot,
+            group_snapshot.GroupSnapshot,
+            False,
+        )
+
+    def test_group_snapshot_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_group_snapshot,
+            group_snapshot.GroupSnapshot,
+            True,
+        )
+
+
 class TestGroupType(TestVolumeProxy):
     def test_group_type_get(self):
         self.verify_get(self.proxy.get_group_type, group_type.GroupType)
@@ -249,6 +330,53 @@ class TestGroupType(TestVolumeProxy):
         )
 
 
+class TestService(TestVolumeProxy):
+    def test_services(self):
+        self.verify_list(self.proxy.services, service.Service)
+
+    def test_enable_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.enable',
+            self.proxy.enable_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_disable_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.disable',
+            self.proxy.disable_service,
+            method_args=["value"],
+            expected_kwargs={"reason": None},
+            expected_args=[self.proxy],
+        )
+
+    def test_thaw_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.thaw',
+            self.proxy.thaw_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_freeze_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.freeze',
+            self.proxy.freeze_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_failover_service(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.failover',
+            self.proxy.failover_service,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_kwargs={"backend_id": None, "cluster": None},
+        )
+
+
 class TestExtension(TestVolumeProxy):
     def test_extensions(self):
         self.verify_list(self.proxy.extensions, extension.Extension)
@@ -295,6 +423,32 @@ class TestVolumeActions(TestVolumeProxy):
             expected_args=[self.proxy, '1', '2', '3'],
         )
 
+    def test_set_volume_image_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.set_image_metadata",
+            self.proxy.set_volume_image_metadata,
+            method_args=["value"],
+            method_kwargs={'foo': 'bar'},
+            expected_args=[self.proxy],
+            expected_kwargs={'metadata': {'foo': 'bar'}},
+        )
+
+    def test_delete_volume_image_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete_image_metadata",
+            self.proxy.delete_volume_image_metadata,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_delete_volume_image_metadata__with_keys(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete_image_metadata_item",
+            self.proxy.delete_volume_image_metadata,
+            method_args=["value", ['foo']],
+            expected_args=[self.proxy, 'foo'],
+        )
+
     def test_volume_revert_to_snapshot(self):
         self._verify(
             "openstack.block_storage.v3.volume.Volume.revert_to_snapshot",
@@ -504,7 +658,11 @@ class TestBackup(TestVolumeProxy):
         # NOTE: mock has_service
         self.proxy._connection = mock.Mock()
         self.proxy._connection.has_service = mock.Mock(return_value=True)
-        self.verify_find(self.proxy.find_backup, backup.Backup)
+        self.verify_find(
+            self.proxy.find_backup,
+            backup.Backup,
+            expected_kwargs={'list_base_path': '/backups/detail'},
+        )
 
     def test_backup_delete(self):
         # NOTE: mock has_service
@@ -560,7 +718,15 @@ class TestSnapshot(TestVolumeProxy):
         self.verify_get(self.proxy.get_snapshot, snapshot.Snapshot)
 
     def test_snapshot_find(self):
-        self.verify_find(self.proxy.find_snapshot, snapshot.Snapshot)
+        self.verify_find(
+            self.proxy.find_snapshot,
+            snapshot.Snapshot,
+            method_kwargs={'all_projects': True},
+            expected_kwargs={
+                'list_base_path': '/snapshots/detail',
+                'all_projects': True,
+            },
+        )
 
     def test_snapshots_detailed(self):
         self.verify_list(
@@ -658,6 +824,24 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy, "key"],
         )
 
+    def test_manage_snapshot(self):
+        kwargs = {
+            "volume_id": "fake_id",
+            "remote_source": "fake_volume",
+            "snapshot_name": "fake_snap",
+            "description": "test_snap",
+            "property": {"k": "v"},
+        }
+        self._verify(
+            "openstack.block_storage.v3.snapshot.Snapshot.manage",
+            self.proxy.manage_snapshot,
+            method_kwargs=kwargs,
+            method_result=snapshot.Snapshot(id="fake_id"),
+            expected_args=[self.proxy],
+            expected_kwargs=kwargs,
+            expected_result=snapshot.Snapshot(id="fake_id"),
+        )
+
 
 class TestType(TestVolumeProxy):
     def test_type_get(self):
@@ -746,11 +930,27 @@ class TestType(TestVolumeProxy):
         )
 
     def test_type_encryption_update(self):
+        # Verify that the get call was made with correct kwargs
+        self.verify_get(
+            self.proxy.get_type_encryption,
+            type.TypeEncryption,
+            method_args=['value'],
+            expected_args=[],
+            expected_kwargs={'volume_type_id': 'value', 'requires_id': False},
+        )
         self.verify_update(
             self.proxy.update_type_encryption, type.TypeEncryption
         )
 
     def test_type_encryption_delete(self):
+        # Verify that the get call was made with correct kwargs
+        self.verify_get(
+            self.proxy.get_type_encryption,
+            type.TypeEncryption,
+            method_args=['value'],
+            expected_args=[],
+            expected_kwargs={'volume_type_id': 'value', 'requires_id': False},
+        )
         self.verify_delete(
             self.proxy.delete_type_encryption, type.TypeEncryption, False
         )
@@ -761,51 +961,82 @@ class TestType(TestVolumeProxy):
         )
 
 
-class TestQuota(TestVolumeProxy):
-    def test_get(self):
+class TestQuotaClassSet(TestVolumeProxy):
+    def test_quota_class_set_get(self):
+        self.verify_get(
+            self.proxy.get_quota_class_set, quota_class_set.QuotaClassSet
+        )
+
+    def test_quota_class_set_update(self):
+        self.verify_update(
+            self.proxy.update_quota_class_set,
+            quota_class_set.QuotaClassSet,
+            False,
+        )
+
+
+class TestQuotaSet(TestVolumeProxy):
+    def test_quota_set_get(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                None,
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
+                'microversion': None,
+                'resource_response_key': None,
                 'usage': False,
             },
             method_result=quota_set.QuotaSet(),
             expected_result=quota_set.QuotaSet(),
         )
 
-    def test_get_query(self):
+    def test_quota_set_get_query(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
             method_kwargs={'usage': True, 'user_id': 'uid'},
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                None,
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
+                'microversion': None,
+                'resource_response_key': None,
                 'usage': True,
                 'user_id': 'uid',
             },
         )
 
-    def test_get_defaults(self):
+    def test_quota_set_get_defaults(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set_defaults,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                '/os-quota-sets/defaults',
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'base_path': '/os-quota-sets/defaults',
+                'microversion': None,
+                'resource_response_key': None,
             },
         )
 
-    def test_reset(self):
+    def test_quota_set_reset(self):
         self._verify(
             'openstack.resource.Resource.delete',
             self.proxy.revert_quota_set,
@@ -815,19 +1046,43 @@ class TestQuota(TestVolumeProxy):
             expected_kwargs={'user_id': 'uid'},
         )
 
-    @mock.patch('openstack.proxy.Proxy._get_resource', autospec=True)
-    def test_update(self, gr_mock):
-        gr_mock.return_value = resource.Resource()
-        gr_mock.commit = mock.Mock()
+    @mock.patch.object(proxy_base.Proxy, '_get_resource')
+    def test_quota_set_update(self, mock_get):
+        fake_project = project.Project(id='prj')
+        mock_get.side_effect = [fake_project]
+
         self._verify(
-            'openstack.resource.Resource.commit',
+            'openstack.proxy.Proxy._update',
             self.proxy.update_quota_set,
-            method_args=['qs'],
-            method_kwargs={
-                'query': {'user_id': 'uid'},
-                'a': 'b',
-            },
-            expected_args=[self.proxy],
-            expected_kwargs={'user_id': 'uid'},
+            method_args=['prj'],
+            method_kwargs={'volumes': 123},
+            expected_args=[quota_set.QuotaSet, None],
+            expected_kwargs={'project_id': 'prj', 'volumes': 123},
         )
-        gr_mock.assert_called_with(self.proxy, quota_set.QuotaSet, 'qs', a='b')
+        mock_get.assert_called_once_with(project.Project, 'prj')
+
+    @mock.patch.object(proxy_base.Proxy, "_get_resource")
+    def test_quota_set_update__legacy(self, mock_get):
+        fake_quota_set = quota_set.QuotaSet(project_id='prj')
+        mock_get.side_effect = [fake_quota_set]
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+
+            self._verify(
+                'openstack.resource.Resource.commit',
+                self.proxy.update_quota_set,
+                method_args=[fake_quota_set],
+                method_kwargs={'ram': 123},
+                expected_args=[self.proxy],
+                expected_kwargs={},
+            )
+
+            self.assertEqual(1, len(w))
+            self.assertEqual(
+                os_warnings.RemovedInSDK50Warning,
+                w[-1].category,
+            )
+            self.assertIn(
+                "The signature of 'update_quota_set' has changed ",
+                str(w[-1]),
+            )
