@@ -24,7 +24,6 @@ EXAMPLE = {
 
 
 class TestServerIP(base.TestCase):
-
     def test_basic(self):
         sot = server_ip.ServerIP()
         self.assertEqual('addresses', sot.resources_key)
@@ -46,29 +45,36 @@ class TestServerIP(base.TestCase):
         resp = mock.Mock()
         sess.get.return_value = resp
         resp.json.return_value = {
-            "addresses": {"label1": [{"version": 1, "addr": "a1"},
-                                     {"version": 2, "addr": "a2"}],
-                          "label2": [{"version": 3, "addr": "a3"},
-                                     {"version": 4, "addr": "a4"}]}}
+            "addresses": {
+                "label1": [
+                    {"version": 1, "addr": "a1"},
+                    {"version": 2, "addr": "a2"},
+                ],
+                "label2": [
+                    {"version": 3, "addr": "a3"},
+                    {"version": 4, "addr": "a4"},
+                ],
+            }
+        }
 
         ips = list(server_ip.ServerIP.list(sess, server_id=IDENTIFIER))
 
         self.assertEqual(4, len(ips))
         ips = sorted(ips, key=lambda ip: ip.version)
 
-        self.assertEqual(type(ips[0]), server_ip.ServerIP)
+        self.assertIsInstance(ips[0], server_ip.ServerIP)
         self.assertEqual(ips[0].network_label, "label1")
         self.assertEqual(ips[0].address, "a1")
         self.assertEqual(ips[0].version, 1)
-        self.assertEqual(type(ips[1]), server_ip.ServerIP)
+        self.assertIsInstance(ips[1], server_ip.ServerIP)
         self.assertEqual(ips[1].network_label, "label1")
         self.assertEqual(ips[1].address, "a2")
         self.assertEqual(ips[1].version, 2)
-        self.assertEqual(type(ips[2]), server_ip.ServerIP)
+        self.assertIsInstance(ips[2], server_ip.ServerIP)
         self.assertEqual(ips[2].network_label, "label2")
         self.assertEqual(ips[2].address, "a3")
         self.assertEqual(ips[2].version, 3)
-        self.assertEqual(type(ips[3]), server_ip.ServerIP)
+        self.assertIsInstance(ips[3], server_ip.ServerIP)
         self.assertEqual(ips[3].network_label, "label2")
         self.assertEqual(ips[3].address, "a4")
         self.assertEqual(ips[3].version, 4)
@@ -78,22 +84,24 @@ class TestServerIP(base.TestCase):
         sess = mock.Mock()
         resp = mock.Mock()
         sess.get.return_value = resp
-        resp.json.return_value = {label: [{"version": 1,
-                                           "addr": "a1"},
-                                          {"version": 2,
-                                           "addr": "a2"}]}
+        resp.json.return_value = {
+            label: [{"version": 1, "addr": "a1"}, {"version": 2, "addr": "a2"}]
+        }
 
-        ips = list(server_ip.ServerIP.list(sess, server_id=IDENTIFIER,
-                                           network_label=label))
+        ips = list(
+            server_ip.ServerIP.list(
+                sess, server_id=IDENTIFIER, network_label=label
+            )
+        )
 
         self.assertEqual(2, len(ips))
         ips = sorted(ips, key=lambda ip: ip.version)
 
-        self.assertEqual(type(ips[0]), server_ip.ServerIP)
+        self.assertIsInstance(ips[0], server_ip.ServerIP)
         self.assertEqual(ips[0].network_label, label)
         self.assertEqual(ips[0].address, "a1")
         self.assertEqual(ips[0].version, 1)
-        self.assertEqual(type(ips[1]), server_ip.ServerIP)
+        self.assertIsInstance(ips[1], server_ip.ServerIP)
         self.assertEqual(ips[1].network_label, label)
         self.assertEqual(ips[1].address, "a2")
         self.assertEqual(ips[1].version, 2)
